@@ -1,5 +1,4 @@
 import settings
-from random import choice
 from worlds.AutoWorld import World, WebWorld
 from worlds.generic.Rules import add_rule
 from typing import Union, Tuple, List, Dict, Set, ClassVar, Mapping, Any
@@ -7,16 +6,15 @@ from .Options import SM64HackOptions
 
 from .Items import SM64HackItem, item_is_important
 from .Locations import SM64HackLocation, location_names, location_names_that_exist
-from .Data import sm64hack_items, star_like, traps, badges, sr6_25_locations, Data
+from .Data import sm64hack_items, star_like, traps, badges, sr6_25_locations, create_json_folders, Data
 from .client import SM64HackClient
+from settings import get_settings
 from BaseClasses import Region, Location, Entrance, Item, ItemClassification, CollectionState, Tutorial
 
-#class SM64HackSettings(settings.Group):
-#    pass
-    #class RomFile(settings.HackRomPath):
-    #    """Insert help text for host.yaml here"""
-    #rom_file: RomFile = RomFile("SM64Hack.z64")
-
+class SM64HackSettings(settings.Group):
+    class AutoUpdate(settings.Bool):
+        """Automatically download updated json files from GitHub when generating SM64Hack worlds"""
+    auto_update: AutoUpdate | bool = True
 
 class SM64HackWebWorld(WebWorld):
     display_name = "Super Mario 64 Romhacks"
@@ -45,7 +43,7 @@ class SM64HackWorld(World):
     
     options_dataclass = SM64HackOptions
     options: SM64HackOptions
-#    settings: ClassVar[SM64HackSettings]
+    settings: ClassVar[SM64HackSettings]
     topology_present = True
     data: Data
     web = SM64HackWebWorld()
@@ -64,12 +62,21 @@ class SM64HackWorld(World):
         super().__init__(multiworld, player)
         self.data = Data()
         
+    @classmethod
+    def stage_assert_generate(cls, multiworld): # this is supposed to be used for rom files but its the only bit of code that i could find that runs before everything and only once before generation so im using it
+        create_json_folders(get_settings()["sm64hacks_options"]["auto_update"] and not hasattr(multiworld, "generation_is_fake"))
+
     def generate_early(self):
-        if isinstance(self.options.json_file.value, int):
-            fn = self.normalize_to_json_filename(self.options.json_file.current_key)
-        else:
-            fn = self.options.json_file.value
-        self.data.import_json(fn)
+        # Get filename from TextChoice option value
+        json_filename = self.options.json_file.get_filename_from_value(self.options.json_file.value)
+        if json_filename is None:
+            # Fallback: try using value as string (for custom values or legacy support)
+            if isinstance(self.options.json_file.value, str):
+                json_filename = self.options.json_file.value
+            else:
+                # Try to get from current_key
+                json_filename = self.options.json_file.current_key + ".json"
+        self.data.import_json(json_filename, self.settings.auto_update)
         self.progressive_keys = self.options.progressive_keys.value
         if isinstance(self.data.locations["Other"]["Settings"], list):
             raise ValueError("JSON is too old. \
@@ -85,43 +92,42 @@ class SM64HackWorld(World):
         self.options.non_local_items.value |= set(non_local_traps)
 
 
-    @staticmethod
-    def normalize_to_json_filename(name: str) -> str:
-        """
-        Replace spaces with underscores and ensure the filename ends with .json 
-        (case-insensitive) if hack was selected on webworld frontend.
-        """
-        # 1. Replace _
-        sanitized = name.replace('_', '').replace('dot', '.')
-        # 2. Check for .json suffix (case-insensitive)
-        if not sanitized.lower().endswith('.json'):
-            sanitized += '.json'
-        return sanitized
-
-    def create_item(self, item: str) -> SM64HackItem:
-        if item == "Power Star":
-            if self.stars_created < self.data.maxstarcount: #only create progression stars up to the max starcount for the hack
+    def create_item(self, item: str, item_link = True) -> SM64HackItem:
+        
+        if item_link: #item link is dumb and i need to make all item_link items some sort of progression
+            classification = ItemClassification.progression
+            if item == "Power Star":
                 classification = ItemClassification.progression_deprioritized_skip_balancing
+        else:
+            if item == "Power Star":
+                if self.stars_created < self.data.maxstarcount: #only create progression stars up to the max starcount for the hack
+                    classification = ItemClassification.progression_deprioritized_skip_balancing
+                    self.stars_created += 1
+                else:
+                    classification = ItemClassification.useful
+            elif item in traps:
+                classification = ItemClassification.trap
+            elif item == "Coin":
+                classification = ItemClassification.filler
+            elif item.endswith("Star"): # cannon stars in sr6.25
+                classification = ItemClassification.progression
                 self.stars_created += 1
             else:
-                classification = ItemClassification.useful
-        elif item in traps:
-            classification = ItemClassification.trap
-        elif item == "Coin":
-            classification = ItemClassification.filler
-        elif item.endswith("Star"): # cannon stars in sr6.25
-            classification = ItemClassification.progression
-            self.stars_created += 1
-        else:
-            classification = ItemClassification.progression if item_is_important(item, self.data) else ItemClassification.useful
+                classification = ItemClassification.progression if item_is_important(item, self.data) else ItemClassification.useful
 
-        if hasattr(self.multiworld, "generation_is_fake") and classification == ItemClassification.useful: #UT shenanigans
-            classification = ItemClassification.progression if item != "Power Star" else ItemClassification.progression_deprioritized_skip_balancing
+            if hasattr(self.multiworld, "generation_is_fake") and classification == ItemClassification.useful: #UT shenanigans
+                classification = ItemClassification.progression if item != "Power Star" else ItemClassification.progression_deprioritized_skip_balancing
         return SM64HackItem(item, classification, self.item_name_to_id[item], self.player)
 
     def create_event(self, event: str):
         return SM64HackItem(event, ItemClassification.progression, None, self.player)
 
+    def get_filler_item_name(self):
+        if self.random.randrange(0, 100) < self.options.filler_trap_percentage:
+            return self.random.choice(traps)
+        return "Coin"
+        
+
     def create_items(self) -> None:
         
         
@@ -149,52 +155,50 @@ class SM64HackWorld(World):
             if(course == "Other"):
                 continue
             if(self.data.locations[course]["Cannon"]["exists"]):
-                self.multiworld.itempool += [self.create_item(f"{course} Cannon")]
+                self.multiworld.itempool += [self.create_item(f"{course} Cannon", False)]
             if(self.data.locations[course]["Troll Star"]["exists"]):
                 junk += 1
             for i in range(8):
                 if self.data.locations[course]["Stars"][i]["exists"]:
                     if "sr6.25" not in self.data.locations["Other"]["Settings"] or (i != 7 or (course != "Course 1" and course != "Bowser 3")): #cannon star nonsense
-                        self.multiworld.itempool += [self.create_item("Power Star")]
+                        self.multiworld.itempool += [self.create_item("Power Star", False)]
         if self.progressive_keys > 0:
             for Key in range(2):
                 if self.data.locations["Other"]["Stars"][Key]["exists"]:
-                    self.multiworld.itempool += [self.create_item("Progressive Key")]
+                    self.multiworld.itempool += [self.create_item("Progressive Key", False)]
         else:
             for Key in range(2):
                 if self.data.locations["Other"]["Stars"][Key]["exists"]:
-                    self.multiworld.itempool += [self.create_item(sm64hack_items[Key])]
+                    self.multiworld.itempool += [self.create_item(sm64hack_items[Key], False)]
         
         for item in range(2,5):
             if self.data.locations["Other"]["Stars"][item]["exists"]:
-                self.multiworld.itempool += [self.create_item(sm64hack_items[item])]
-        
-        if(self.options.randomize_moat):
-            if self.data.locations["Other"]["Stars"][5]["exists"]:
-                self.multiworld.itempool += [self.create_item("Castle Moat")]
+                self.multiworld.itempool += [self.create_item(sm64hack_items[item], False)]
         
         if("sr7" in self.data.locations["Other"]["Settings"]):
             for item in range(5):
                 if item < 2:
                     if self.data.locations["Other"]["Stars"][item + 7]["exists"]:
-                        self.multiworld.itempool += [self.create_item("Progressive Stomp Badge")]
+                        self.multiworld.itempool += [self.create_item("Progressive Stomp Badge", False)]
                 else:
                     if self.data.locations["Other"]["Stars"][item + 7]["exists"]:
-                        self.multiworld.itempool += [self.create_item(sm64hack_items[item + 32])]
+                        self.multiworld.itempool += [self.create_item(sm64hack_items[item + 32], False)]
 
         if("sr6.25" in self.data.locations["Other"]["Settings"]):
-            self.multiworld.itempool += [self.create_item("Yellow Switch")]
-            self.multiworld.itempool += [self.create_item("Overworld Cannon Star")]
-            self.multiworld.itempool += [self.create_item("Bowser 2 Cannon Star")]
-        
-        if("sr3.5" in self.data.locations["Other"]["Settings"]):
-            self.multiworld.itempool += [self.create_item("Black Switch")]
+            self.multiworld.itempool += [self.create_item("Yellow Switch", False)]
+            self.multiworld.itempool += [self.create_item("Overworld Cannon Star", False)]
+            self.multiworld.itempool += [self.create_item("Bowser 2 Cannon Star", False)]
+        elif("sr3.5" in self.data.locations["Other"]["Settings"]):
+            self.multiworld.itempool += [self.create_item("Black Switch", False)]
+        elif(self.options.randomize_moat):
+            if self.data.locations["Other"]["Stars"][5]["exists"]:
+                self.multiworld.itempool += [self.create_item("Castle Moat", False)]
         #print("TEST" + str(len(self.multiworld.itempool)))
 
         if self.options.troll_stars == 1:
-            self.multiworld.itempool += [self.create_item(choice(traps)) for _ in range(junk)]
+            self.multiworld.itempool += [self.create_item(self.random.choice(traps), False) for _ in range(junk)]
         elif self.options.troll_stars == 2:
-            self.multiworld.itempool += [self.create_item("Coin") for _ in range(junk)]
+            self.multiworld.itempool += [self.create_item("Coin", False) for _ in range(junk)]
 
 
 
@@ -330,8 +334,15 @@ class SM64HackWorld(World):
                     star_data = self.data.locations[course]["Stars"][item]
                     if(star_data.get("exists")):
                         if item == 5:
-                            add_rule(self.multiworld.get_location("Castle Moat", self.player),
-                                     lambda state, star_data=self.data.locations[course]["Stars"][item]: self.can_access_location(state, star_data))
+                            if("sr6.25" in self.data.locations["Other"]["Settings"]):
+                                add_rule(self.multiworld.get_location("Yellow Switch", self.player),
+                                        lambda state, star_data=self.data.locations[course]["Stars"][5]: self.can_access_location(state, star_data))
+                            elif("sr3.5" in self.data.locations["Other"]["Settings"]):
+                                add_rule(self.multiworld.get_location("Black Switch", self.player),
+                                        lambda state, star_data=self.data.locations[course]["Stars"][5]: self.can_access_location(state, star_data))
+                            else:
+                                add_rule(self.multiworld.get_location("Castle Moat", self.player),
+                                        lambda state, star_data=self.data.locations[course]["Stars"][item]: self.can_access_location(state, star_data))
                         else:
                             add_rule(self.multiworld.get_location(sm64hack_items[item], self.player),
                                 lambda state, star_data=self.data.locations[course]["Stars"][item]: self.can_access_location(state, star_data))
@@ -343,17 +354,9 @@ class SM64HackWorld(World):
                     for item in range(5):
                         stardata = self.data.locations[course]["Stars"][item + 7]
                         if(stardata.get("exists")):
-                            print(self.multiworld.get_location(badges[item], self.player))
                             add_rule(self.multiworld.get_location(badges[item], self.player),
                                 lambda state, star_data=self.data.locations[course]["Stars"][item + 7]: self.can_access_location(state, star_data))
                 
-                if("sr6.25" in self.data.locations["Other"]["Settings"]):
-                    add_rule(self.multiworld.get_location("Yellow Switch", self.player),
-                             lambda state, star_data=self.data.locations[course]["Stars"][5]: self.can_access_location(state, star_data))
-                
-                if("sr3.5" in self.data.locations["Other"]["Settings"]):
-                    add_rule(self.multiworld.get_location("Black Switch", self.player),
-                             lambda state, star_data=self.data.locations[course]["Stars"][5]: self.can_access_location(state, star_data))
 
                 star_data = self.data.locations[course]["Stars"][6]
                 add_rule(self.multiworld.get_location("Victory Location", self.player),
@@ -392,8 +395,10 @@ class SM64HackWorld(World):
     
     def generate_basic(self) -> None:
         self.multiworld.get_location("Victory Location", self.player).place_locked_item(self.create_event("Victory"))
-        if not self.options.randomize_moat.value and self.data.locations["Other"]["Stars"][5]["exists"]:
-            self.multiworld.get_location("Castle Moat", self.player).place_locked_item(self.create_item("Castle Moat"))
+        if not self.options.randomize_moat.value and self.data.locations["Other"]["Stars"][5]["exists"] \
+                and not "sr6.25" in self.data.locations["Other"]["Settings"] \
+                and not "sr3.5" in self.data.locations["Other"]["Settings"]:
+            self.multiworld.get_location("Castle Moat", self.player).place_locked_item(self.create_item("Castle Moat", False))
         self.multiworld.completion_condition[self.player] = lambda state: state.has("Victory", self.player)
 
     def fill_slot_data(self) -> Mapping[str, Any]:
